@@ -333,6 +333,13 @@ int main(int argc, char **argv)
 	rc = phase0_processing();
 	if (agg_recptr->fsck_is_done)
 		goto phases_complete;
+	/*
+	 * If -n flag was specified, disable write processing now
+	 */
+	if (agg_recptr->parm_options[UFS_CHKDSK_LEVEL0]) {
+		agg_recptr->processing_readonly = 1;
+		agg_recptr->processing_readwrite = 0;
+	}
 	rc = phase1_processing();
 	if (agg_recptr->fsck_is_done)
 		goto phases_complete;
@@ -2982,8 +2989,6 @@ int validate_fs_inodes()
  */
 int verify_parms()
 {
-	int mutexcl_parm_count = 0;
-
 	/*
 	 * If -f was chosen, have it override -a, -p, -r by
 	 * turning off UFS_CHKDSK_IFDIRTY to force a check
@@ -2996,40 +3001,36 @@ int verify_parms()
 	}
 
 	/*
-	 * Verify that only one of the mutually exclusive parms was specified
-	 * Set default value if none was specified.
+	 * If the -n flag was specified, turn off -p
 	 */
-	mutexcl_parm_count = agg_recptr->parm_options[UFS_CHKDSK_LEVEL0] +
-	    agg_recptr->parm_options[UFS_CHKDSK_LEVEL2];
-	switch (mutexcl_parm_count) {
-	case 0:
-		/* set the default to same as -p */
+	if (agg_recptr->parm_options[UFS_CHKDSK_LEVEL0]) {
+		/*
+		 * If -n is specified by itself, don't replay the journal.
+		 * If -n is specified with -a, -p, or -f, replay the journal
+		 * but don't make any other changes
+		 */
+		if (agg_recptr->parm_options[UFS_CHKDSK_LEVEL2] == 0)
+			agg_recptr->parm_options[UFS_CHKDSK_SKIPLOGREDO] = -1;
+		agg_recptr->parm_options[UFS_CHKDSK_LEVEL2] = -1;
+	}
+	else if (agg_recptr->parm_options[UFS_CHKDSK_LEVEL2]) {
+		agg_recptr->parm_options[UFS_CHKDSK_LEVEL1] = 0;
+		agg_recptr->parm_options[UFS_CHKDSK_LEVEL2] = 0;
+		agg_recptr->parm_options[UFS_CHKDSK_LEVEL3] = -1;
+		/*
+		 * we'll be doing the Bad Block List function as part
+		 * of the -f processing.  Turn off the flag that specifies
+		 * it.  This flag is only used when we're in read-only mode.
+		 */
+		agg_recptr->parm_options[UFS_CHKDSK_CLRBDBLKLST] = 0;
+	} else {
+		/*
+		 * Set default value if none was specified.
+		 * set the default to same as -p
+		 */
 		agg_recptr->parm_options[UFS_CHKDSK_LEVEL2] = -1;
 		agg_recptr->parm_options[UFS_CHKDSK_IFDIRTY] = -1;
 		fsck_send_msg(fsck_SESSPRMDFLT);
-		break;
-	case (-1):
-		/* exactly one was specified */
-
-		/*
-		 * assume permission to fix everything
-		 */
-		if (!agg_recptr->parm_options[UFS_CHKDSK_LEVEL0]) {
-			/* not read only */
-			agg_recptr->parm_options[UFS_CHKDSK_LEVEL1] = 0;
-			agg_recptr->parm_options[UFS_CHKDSK_LEVEL2] = 0;
-			agg_recptr->parm_options[UFS_CHKDSK_LEVEL3] = -1;
-			/*
-			 * we'll be doing the Bad Block List function as part
-			 * of the -f processing.  Turn off the flag that specifies
-			 * it.  This flag is only used when we're in read-only mode.
-			 */
-			agg_recptr->parm_options[UFS_CHKDSK_CLRBDBLKLST] = 0;
-		}
-		break;
-	default:
-		fsck_send_msg(fsck_PRMMUTEXCLLEVELS);
-		return (FSCK_FAILED);
 	}
 	/*
 	 * the parms are valid.  record the messaging level they imply.
